@@ -569,7 +569,6 @@ ParseXpattern.prototype.drawCreateTab = function(surface, win, context) {
   xmlHelper.emptyChildren(surface);
   //dump("drawCreateTab start\n");
 
-
   var vbox = xmlHelper.appendNode(surface, "vbox", null, {flex:1}, null);
 
   var hbox = xmlHelper.appendNode(surface, "hbox", null, null, null);
@@ -672,6 +671,7 @@ ParseXpattern.prototype.drawCreateTab = function(surface, win, context) {
     context.setPattern("");
     addNodeButton.hidden = false; // show the initial "add" button
     alertmsg.hidden = true; // hide old "too complex" message
+    waitmsg.hidden = true; // hide old "recalculating" message
     var pageDoc = context.getPageDoc(); // unhighlight all nodes
     var hitarea = context.conf['hitarea'];
     var node = xmlHelper.getElementByNodeSpec(pageDoc, hitarea);
@@ -693,10 +693,19 @@ ParseXpattern.prototype.drawCreateTab = function(surface, win, context) {
       }, false);
   }; // clearMenu
 
-  function cardinalityMenu ( menu, line ) {
-    var cardmenu = xulHelper.menu( menu, "Cardinality" );
-    var cardmenupoint = function ( label, cardinality ) {
-      var menupoint = xulHelper.menuitem(cardmenu,label+" "+cardinality);
+  // Cardinality menu. pulldown should be true for pulldown menus,
+  // false (or missing) for menu points in deeper menu structures
+  function cardinalityMenu ( menu, line, pulldown ) {
+    var cardmenu;
+    if ( ! pulldown )
+      cardmenu = xulHelper.menu( menu, "Cardinality" );
+    else
+      cardmenu = menu;
+    var cardmenupoint = function ( label, cardinality, attrs ) {
+      var attrs = { type:"radio", name:"card"};
+      if ( line.cf_cardinality == cardinality )
+        attrs.checked = true;
+      var menupoint = xulHelper.menuitem(cardmenu,label+" "+cardinality, attrs);
         menupoint.addEventListener("click", function(e) {
           line.cf_cardinality = cardinality;
           line.redraw();
@@ -711,12 +720,20 @@ ParseXpattern.prototype.drawCreateTab = function(surface, win, context) {
     cardmenupoint("Repeating optional non-greedy", "*?");
   }; // drawCreateTab.cardinalityMenu
 
-  function variableMenu (menu, line ) {
+  function variableMenu (menu, line, pulldown ) {
     // Helper to make one menu point
-    var varmenu = xulHelper.menu( menu, "Variable" );
+    var varmenu;
+    if ( ! pulldown )
+      varmenu = xulHelper.menu( menu, "Variable" );
+    else {
+      varmenu = menu;
+    }
     function varmenupoint ( varname, dispname ) {
       if (!dispname) dispname = varname;
-      var menupoint = xulHelper.menuitem(varmenu,dispname);
+      var attrs = { type:"radio", name:"var"};
+      if ( line.cf_varname == varname )
+        attrs.checked = true;
+      var menupoint = xulHelper.menuitem(varmenu,dispname, attrs);
         menupoint.addEventListener("click", function(e) {
           line.cf_varname = varname;
           line.redraw();
@@ -983,6 +1000,7 @@ ParseXpattern.prototype.drawCreateTab = function(surface, win, context) {
   function selectAndAddField(box, addAfter ) {
     pleasemsg.hidden = false;
     alertmsg.hidden = true;
+    waitmsg.hidden = true;
     app.newHl(context.getPageDoc(), function() {
         pleasemsg.hidden = true;
         var domNode = this;
@@ -1003,7 +1021,112 @@ ParseXpattern.prototype.drawCreateTab = function(surface, win, context) {
       } ); // highlighter
   }; // selectAndAddField
 
-  
+  // Helpers for guessing a $variable
+
+  // Get to the root of the pattern, or at least to the root of the
+  // current alternative
+  function altroot(fieldbox) {
+    while (fieldbox) {
+      logger.debug("altroot: " + fieldbox.nodeName + ":" +
+        ( fieldbox.cf_tagname || "" ) );
+      if ( fieldbox == nodebox )
+        return fieldbox; // root of the designer tree
+      if ( fieldbox.cf_groupType  && fieldbox.cf_groupType == "ALT" )
+        return fieldbox;
+      fieldbox = fieldbox.parentNode;
+    }
+    return null;  // should not happen
+  };
+
+  // Check if the given variable is already used (within this alternative)
+  // recurses the whole xul domtree from altroot down, this catches groups,
+  // attributes, and everything else that might have a cf_varname. 
+  function variableAlreadyUsed(altroot, varname) {
+    if (!varname)
+      return false;
+    while ( altroot ) {
+      if ( altroot.cf_varname && altroot.cf_varname == varname )
+        return true;
+      if ( variableAlreadyUsed(altroot.firstChild, varname) )
+        return true;
+      altroot = altroot.nextSibling;
+    }
+    return false;
+  };
+
+  // Guess a variable for a newly added line
+  function guessVar(fieldbox, txt) {
+    var root = altroot(fieldbox); // root of the current alternative
+    if ( !root )
+      return "";  // should not happen
+    context.varnamelist();
+    logger.debug("guessVar: varnames: " + JSON.stringify(varnames) );
+    var vars = {};
+    for (var i=0; i<varnames.length; i++) {
+      vars[ varnames[i] ] = 0;
+    }
+    vars[""] = -1; // "none"
+    
+    var hints = context.task.getTemplate().properties.variablehints;
+    if ( hints ) {  // may not be, if not in parse task
+      for ( var h = 0; h < hints.length; h++ ) {
+        var hint = hints[h];
+        logger.debug("guessVar: Looking at a hint for " + hint.var );
+        for ( var v in vars ) {
+          if ( v.match( hint.var ) ) {
+            logger.debug("guessVar: checking '" + v + "' against " + hint.var );
+            if ( hint.regexp ) {
+              var re = new RegExp(hint.regexp, 'i');  // ignore case in matching text
+              if ( txt.match(re) ) {
+                vars[v] += parseInt(hint.weight);
+                logger.debug("guessVar: Regexp " + hint.regexp + " matches '" +
+                 txt + "'.  Adjusted hint for " + v + " by " +
+                  hint.weight + ". Now at " + vars[v] );
+              } else {
+                logger.debug("guessVar: Regexp " + hint.regexp + " did not match " +
+                 "'" + txt + "'" );
+              }
+            } else if ( hint.func ) {
+              if ( hint.func == "alreadyused" ) {
+                if ( variableAlreadyUsed(root.firstChild, v) ) {
+                  vars[v] += parseInt(hint.weight);
+                  logger.debug("guessVar: alreadyused. Adjusted " + v + " by " +
+                    hint.weight + " now at " + vars[v] );
+                }
+              }
+            } else if ( hint.attr ) {
+              if ( hint.attr == fieldbox.cf_tagname ) {
+                vars[v] += parseInt(hint.weight);
+                logger.debug("guessVar: attr " + hint.attr + " matches '" +
+                 txt + "'.  Adjusted hint for " + v + " by " +
+                  hint.weight + ". Now at " + vars[v] );                
+              }
+
+            } else
+              logger.error("Bad variable hint: Need a regexp or a func. " +
+                JSON.stringify(hint) );
+          }
+        }
+
+      }
+    }
+    var bestvar = varnames[0];  // the first in the list, usually title
+    for (var v in vars ) {
+      if ( vars[v] > vars[ bestvar ] ) {
+        bestvar = v;
+      }
+      logger.debug("guessVar: " + v + ": " + vars[v] + " best=" + bestvar);
+    }
+    if ( vars[bestvar] >= 0 ) {
+      logger.debug("guessVar: Decided to guess '" + bestvar + "' " +
+        " with a score of " + vars[bestvar] );
+      var ret = { guess: bestvar, score: vars[bestvar] };
+      return ret ;
+    }
+
+    return "";  // didn't find anything
+  }// guessVar
+
   // Add a line that displays a one field, and possibly its attributes too
   function addField ( domNode, pattNode, box, hlColor, addAfter ) {
     //logger.debug("addField: dom=" + domNode + " patt=" + pattNode +
@@ -1028,39 +1151,53 @@ ParseXpattern.prototype.drawCreateTab = function(surface, win, context) {
     fieldBox.cf_domNode = domNode; // keep a reference to it, for generating
     fieldBox.cf_lineType = "field";
     fieldBox.cf_tagname = domNode.localName || domNode.nodeName;
-    fieldBox.cf_text = domNode.textContent;
+    fieldBox.cf_text = domNode.textContent || "<" + fieldBox.cf_tagname + ">";
+       // IMG tags have no text, so display the tag instead
     fieldBox.cf_cardinality = "";
     fieldBox.cf_varname = "";
     fieldBox.cf_modifiers = [];
     fieldBox.cf_highlight = hlColor;
     fieldBox.cf_collapsed = false;
+    var varguessscore = 0;
     if ( pattNode ) {
       fieldBox.cf_cardinality = pattNode.getCardinality();
       fieldBox.cf_varname = pattNode.getPlainVariable();
       fieldBox.cf_modifiers = pattNode.getModifiers();
+    } else {
+      var g = guessVar(fieldBox, domNode.textContent );
+      if ( g ) {
+        fieldBox.cf_varname = g.guess;
+        varguessscore = g.score;
+      }
     }
-    fieldBox.dispString = function () {
+
+    // helper do redraw the field elements. Called every time things change.
+    fieldBox.redraw = function () {
       var disp = fieldBox.cf_tagname;
       if ( fieldBox.cf_cardinality )
         disp += " " + fieldBox.cf_cardinality;
+      xulHelper.changemenulabel( cardmenu, disp);
       if ( fieldBox.cf_varname )
-        disp += " $" + fieldBox.cf_varname;
+        disp = "$" + fieldBox.cf_varname;
+      else
+        disp = "$";
+      xulHelper.changemenulabel( varmenu, disp);
+      disp = "";
       if ( fieldBox.cf_modifiers )
         for ( var m in fieldBox.cf_modifiers )
           disp += " -" + fieldBox.cf_modifiers[m];
-      if ( fieldBox.cf_attrvalue ) 
-        fieldBox.cf_attrline = "/" + fieldBox.cf_attrvalue + "/";
-      else
-        fieldBox.cf_attrline = "";
+      moddisp.value = disp;
+      disp = "";
       if ( fieldBox.cf_collapsed ) {
-        disp += " ...";
-      } else { // not collapsed
-        if ( fieldBox.cf_attrline ) {
-          disp += " [ " + fieldBox.cf_attrline+ " ]";
-        }
+        disp = "[...]";
+      } else {
+        if ( fieldBox.cf_attrvalue )
+          disp = "[ /" + fieldBox.cf_attrvalue+ "/ ]";
       }
-      return disp;
-    }; // dispString
+      attrdisp.value = disp;
+      checkHitArea();
+    }; // redraw
+
     var txtdisp = xmlHelper.appendNode(fieldLine, "label", null, 
                             { flex:1, crop:"end", value:fieldBox.cf_text } );
         // If the value is passed as a text, it gets multilined, and not cropped
@@ -1068,9 +1205,23 @@ ParseXpattern.prototype.drawCreateTab = function(surface, win, context) {
     var hitwarning = xmlHelper.appendNode(fieldLine, "label", null,
                     { style:"color:red;font-weight:bold;", hidden:true,
                       value: "(hit area!)" } );
-    var disp = fieldBox.dispString();
-    var nodedisp = xmlHelper.appendNode(fieldLine, "label", disp,
-                           { style:"font-weight: bold;" } )
+    var cardmenu = xulHelper.popupmenu(fieldLine, "???",
+                            {style:"font-weight: bold;"} );
+      // ## Wanted it to be underlined as well, but setting
+      // text-decoration: underline caused all menu points to be underlined too.
+      // Anyway, that would conflict with hotkeys, if we ever set such
+    cardinalityMenu(cardmenu, fieldBox, true);
+
+    var varmenu = xulHelper.popupmenu(fieldLine, "???",
+                            {style:"font-weight: bold;"} );
+    variableMenu(varmenu, fieldBox, true);
+    
+    var moddisp =  xmlHelper.appendNode(fieldLine, "label", "???",
+                           { style:"font-weight: bold;" } );
+    var attrdisp =  xmlHelper.appendNode(fieldLine, "label", "???",
+                           { style:"font-weight: bold;" } );
+    
+    fieldBox.redraw();
     
     function checkHitArea() {
       if ( !fieldBox.cf_domNode ) {
@@ -1094,13 +1245,10 @@ ParseXpattern.prototype.drawCreateTab = function(surface, win, context) {
         logger.debug("checkHitArea: WARN");
       }
     }; // checkHitArea
+    
     checkHitArea();
     hitwarning.addEventListener("click",checkHitArea, false);
     
-    fieldBox.redraw = function () {
-      nodedisp.value = this.dispString();
-      checkHitArea();
-    }; // redraw
 
     var toolmenu = xulHelper.menu(fieldLine, "Actions" );
     cardinalityMenu(toolmenu, fieldBox);
@@ -1118,17 +1266,26 @@ ParseXpattern.prototype.drawCreateTab = function(surface, win, context) {
     // little filler at the end of the line, to make attr menus look indented
     xmlHelper.appendNode(fieldLine, "hbox", null, {width:20}, null);
 
-    // collapse/expand the line. 
-    fieldBox.collapse = function () {
-      fieldBox.cf_attrbox.hidden = true;
-      fieldBox.cf_collapsed = true;
-      fieldBox.redraw();
+    if ( !pattNode && varguessscore < 1000 ) { // user-selected field
+      varmenu.openPopup(varmenu.cf_label,"after_end"); // and not certain guess
     }
+
+    // collapse/expand the line.
+    fieldBox.collapse = function () {
+      if ( fieldBox.cf_attrvalue || fieldBox.cf_attrbox.firstChild ) {
+        // only collapse if we have something to collapse
+        fieldBox.cf_attrbox.hidden = true;
+        fieldBox.cf_collapsed = true;
+        fieldBox.redraw();
+      }
+    };
+    
     fieldBox.expand = function () {
       fieldBox.cf_attrbox.hidden = false;
       fieldBox.cf_collapsed = false;
       fieldBox.redraw();
-    }
+    };
+    
     fieldBox.addEventListener("dblclick", function(e) {
         if ( fieldBox.cf_collapsed )
           fieldBox.expand();
@@ -1151,35 +1308,55 @@ ParseXpattern.prototype.drawCreateTab = function(surface, win, context) {
     attrBox.cf_text = domNode.textContent;
     attrBox.cf_highlight = hlColor;
     attrBox.cf_varname = "";
+    var varguessscore = 0;
     if ( pattNode ) {
       attrBox.cf_attrvalue = pattNode.attrValue || "";
       attrBox.cf_attrop = pattNode.relationValue;
       attrBox.cf_varname = pattNode.getPlainVariable();
+    } else {
+      var g = guessVar(attrBox, domNode.textContent );
+      if ( g ) {
+        attrBox.cf_varname = g.guess;
+        varguessscore = g.score;
+      }
     }
 
-    attrBox.dispString = function () {
-      var disp = "@" + attrBox.cf_tagname ;
+    var txtdisp = xmlHelper.appendNode(attrLine, "label", null,
+                         { flex:1, crop:"end", value:attrBox.cf_text } );
+
+    attrBox.redraw = function () {
+      var disp = "@" + attrBox.cf_tagname; 
       if ( attrBox.cf_attrvalue ) {
         if ( attrBox.cf_attrop == "=" )
           disp += " =\"" + attrBox.cf_attrvalue + "\"" ;
         else if ( attrBox.cf_attrop == "~" )
           disp += " ~/" + attrBox.cf_attrvalue + "/" ;
       }
+      attrBox.cf_attrline = disp; // Save for generating
+         // TODO - Don't set real values in display things!
+      xulHelper.changemenulabel( attropmenu, disp);
       if ( attrBox.cf_varname )
-        disp += " $" + attrBox.cf_varname;
-      attrBox.cf_attrline = disp; // Save it for generating the pattern
-      disp = "[ " + disp + " ]";
-      return disp;
-    }; // dispString
-    var disp = attrBox.dispString();
-    var txtdisp = xmlHelper.appendNode(attrLine, "label", null,
-                         { flex:1, crop:"end", value:attrBox.cf_text } );
-    var nodedisp = xmlHelper.appendNode(attrLine, "label", null,
-                           { style:"font-weight: bold;", value:disp } )
-
-    attrBox.redraw = function () {
-      nodedisp.value = this.dispString();
+        disp = "$" + attrBox.cf_varname;
+      else
+        disp = "$";
+      xulHelper.changemenulabel( varmenu, disp);
+      
     }; // redraw
+
+    
+    xmlHelper.appendNode(attrLine, "label", null,
+                           { style:"font-weight: bold;", value:"[" } )
+    
+    var attropmenu = xulHelper.popupmenu(attrLine, "???",
+                            {style:"font-weight: bold;"} );
+    attrOpMenu(attropmenu, attrBox, true);
+    var varmenu = xulHelper.popupmenu(attrLine, "???",
+                            {style:"font-weight: bold;"} );
+    variableMenu(varmenu, attrBox, true);
+    xmlHelper.appendNode(attrLine, "label", null,
+                           { style:"font-weight: bold;", value:"]" } )
+    attrBox.redraw();
+    
     var toolmenu = xulHelper.menu(attrLine, "Actions" );
     attrOpMenu(toolmenu, attrBox);
     matchMenu(toolmenu, attrBox, hlColor );
@@ -1187,6 +1364,20 @@ ParseXpattern.prototype.drawCreateTab = function(surface, win, context) {
     variableMenu(toolmenu, attrBox);
     xulHelper.menuseparator(toolmenu);
     removeMenu(toolmenu, attrBox, "attribute");
+
+    /*
+     // TODO - This does not work. My theory is that we still have the
+     // fieldBox menu open (since add attr comes from there), and the
+     // system can not have two menus open at the same time. Yet I seem
+     // not to be able to close it... Later...
+    if ( !pattNode ) { // user-selected field
+      //logger.debug("Closing the field-level menu");
+      //box.cf_toolmenu.closePopup(); 
+      logger.debug("Popping up the var menu for the attr line" );
+      varmenu.openPopup(varmenu.cf_label,"after_end");
+    }
+    */
+    
   }; // drawcreateTab.addAttrLine
 
 
@@ -1295,20 +1486,50 @@ ParseXpattern.prototype.drawCreateTab = function(surface, win, context) {
       // A bit bigger filler, to move the menu further left, to make regular
       // field lines look indented
 
+    // Helper to recursively mark all DOM nodes as collapsed or not,
+    // so we know (not) to highlight collapsed alternatives
+    // Also marks all lines
+    function collapseMarks(xulNode, is_collapsed) {
+      while(xulNode) {
+        if (xulNode.cf_domNode ) {
+          if ( xulNode.cf_domNode.setAttribute ) {
+            if ( is_collapsed )  
+              xulNode.cf_domNode.setAttribute( "cf_collapsed", "collapsed");
+            else
+              xulNode.cf_domNode.setAttribute( "cf_collapsed", "expanded");
+          }
+          xulNode.cf_collapsed = is_collapsed;
+        }
+        if ( xulNode.firstChild )
+          collapseMarks(xulNode.firstChild, is_collapsed );
+        xulNode = xulNode.nextSibling;
+      }
+    }// collapseMarks
+    
     // collapse/expand the line.
     groupBox.collapse = function () {
       groupBox.cf_contentbox.hidden = true; 
       groupBox.cf_collapsed = true;
       groupBox.redraw();
+      if ( grouptype == "ALT" ) {
+        collapseMarks( groupBox.firstChild, true );
+        generatePattern(true);  // regenerate, to get highlights updated
+      }
     }
     groupBox.expand = function () {
       groupBox.cf_contentbox.hidden = false;
       groupBox.cf_collapsed = false;
       groupBox.redraw();
+      if ( grouptype == "ALT" ) {
+        collapseMarks( groupBox.firstChild, false );
+        generatePattern(true);
+      }
     }
+    
     groupLine.addEventListener("dblclick", function(e) {
         // only groupLine, not to react in the whole box, and nested groups
-        logger.debug("Dbl-click on group box " + groupBox.cf_varname );
+        logger.debug("Dbl-click on group box " + groupBox.cf_groupType +
+           " $" + groupBox.cf_varname );
         if ( groupBox.cf_collapsed )
           groupBox.expand();
         else
@@ -1327,18 +1548,32 @@ ParseXpattern.prototype.drawCreateTab = function(surface, win, context) {
     while (line != null) {
       if ( line.cf_lineType == "field" ) {
         var attrs = "";
-        if ( line.cf_attrline )
-          attrs = line.cf_attrline;
+        if ( line.cf_attrvalue )
+          attrs = "/" + line.cf_attrvalue + "/";
         var attrLine = line.cf_attrbox.firstChild;
         while ( attrLine ) {
-          //logger.debug("makelines: looking attr " + attrLine.cf_attrline );
           if (attrs)
             attrs += ", ";
-          attrs += attrLine.cf_attrline;
+          var disp = "@" + attrLine.cf_tagname;
+          if ( attrLine.cf_attrvalue ) {
+            if ( attrLine.cf_attrop == "=" )
+              disp += " =\"" + attrLine.cf_attrvalue + "\"" ;
+            else if ( attrLine.cf_attrop == "~" ) {
+              disp += " ~/" +attrLine.cf_attrvalue + "/" ;
+              // TODO - The XPattern syntax does not allow escaping slashes,
+              // so we can't protect them here. This goes visibly wrong with
+              // the most common attribute, url. Luckily nobody is likely to
+              // do matching on that!
+            }
+          }
+          if ( attrLine.cf_varname )
+            disp += " $" + attrLine.cf_varname;
+          logger.debug("maker: attr line '" + disp + "'" );
+          attrs += disp;
           attrLine = attrLine.nextSibling;
         }
-        //logger.debug("makerLines: " + line.cf_tagname + " '" + line.cf_text + "' " +
-        //" (g=" + groupid + ") m=" + line.cf_modifiers );
+        logger.debug("makerLines: " + line.cf_tagname + " '" + line.cf_text + "' " +
+          " (g=" + groupid + ") m=" + line.cf_modifiers + " a='" + attrs + "'" );
         maker.addNode( line.cf_domNode, line.cf_varname, line.cf_cardinality,
                        attrs, groupid, line.cf_modifiers );  
       } else if ( line.cf_lineType == "group" ) {
@@ -1369,11 +1604,11 @@ ParseXpattern.prototype.drawCreateTab = function(surface, win, context) {
   // disabled by a config option, and error handling may be different.
   // (still within drawCreateTab)
   function generatePattern(autogen) {
+      alertmsg.hidden = true;
       if ( autogen && ! context.conf["autogenerate"] ) {
         logger.debug("no autogeneration");
-        alertmsg.hidden = true;
         waitmsg.hidden = true;
-        generateButton.hidden = false; 
+        generateButton.hidden = false;
         return;
       }
       logger.debug("About to generate pattern");
@@ -1608,7 +1843,7 @@ ParseXpattern.prototype.drawCreateTab = function(surface, win, context) {
   // Set up the designer from an existing XPattern
   // autogen should be set when populating automatically. Defaults to false.
   function populateDesigner( autogen ) {
-    if ( ! autogen )
+    if ( ! autogen ) // sanitize into a boolean
       autogen = false;
     alertmsg.hidden = true; 
     if ( !context.conf['xpattern'] ) {
@@ -1701,7 +1936,7 @@ ParseXpattern.prototype.drawCreateTab = function(surface, win, context) {
     if ( xpatt && xpatt.textContent != patt ) {
       logger.debug("Updating designer label" );
       if ( trimpat(patt) != trimpat(xpatt.textContent) ) {
-        waitmsg.hidden = false;
+        waitmsg.hidden = false; // signal we need to recalculate
         alertmsg.hidden = true; // only one msg at a time
         logger.debug("Real change in patter, invalidating the designer");
       }
@@ -1882,6 +2117,16 @@ ParseXpattern.prototype.drawHistoryTab = function (surface, context) {
 // Highlight all hits
 ParseXpattern.prototype.highlightHits = function (hitsarray) {
   // find the highlights from the dom tree
+  // If we find just one dom element that is collapsed, we set the
+  // collapsed flag in the pattern node, so we refuse to highlight anything
+  // from that pattern. 
+  for (var i=0; i<hitsarray.length; i++) {
+    var h = hitsarray[i];
+    for (var j=0; j<h.hits.length; j++) {
+      var p = h.hits[j]["pattern"];
+      p.is_collapsed = false; 
+    }
+  }
   for (var i=0; i<hitsarray.length; i++) {
     var h = hitsarray[i];
     for (var j=0; j<h.hits.length; j++) {
@@ -1892,21 +2137,26 @@ ParseXpattern.prototype.highlightHits = function (hitsarray) {
           d = d.parentNode;
         if ( d.nodeName != "#comment" && d.nodeName != "#cdata-section" ) {
           var hi = d.getAttribute("cf_user_highlight");
+          var is_collapsed =  d.getAttribute("cf_collapsed") || "expanded";
           if (hi) {
               p.highlight = hi;
-              xulHelper.highlightNode(d, hi, true );
-              //logger.debug("highlightHits: found hilight " + hi +
-              //   " at " + h.hits[j]["value"] + " p=" + p.dumpString(-2) );
+              if ( is_collapsed == "expanded") {
+                xulHelper.highlightNode(d, hi, true );
+              } else {
+                xulHelper.unhighlightNode(d, false );
+                p.is_collapsed = true;
+                
+              }
           }
           if ( ! p.highlight ) { // set highlight if not yet set
             var col = gethighlightcolor();
             p.highlight = col;
-            xulHelper.highlightNode(d, col, true );
-            /*
-            dump("Assigned highlight N " + col +
-                " to pattern " + p.dumpString(-2) +
-                " and dom node " + xmlHelper.getElementXpath(d) + "\n");
-            */
+            if ( is_collapsed == "expanded" ) {
+              xulHelper.highlightNode(d, col, true );
+            } else {
+              xulHelper.unhighlightNode(d, false );
+              p.is_collapsed = true;
+            }
           }
         }
       }
@@ -1924,18 +2174,25 @@ ParseXpattern.prototype.highlightHits = function (hitsarray) {
             d = d.parentNode;
           }
           if ( d.nodename != "#comment" ) {
-            xulHelper.highlightNode(d, p.highlight, false);
-            //dump("setting highlight " + hi +
-            //     " at " + h.hits[j]["value"] + "\n");
+            //var is_collapsed =  d.getAttribute("cf_collapsed");
+            var is_collapsed =  p.is_collapsed;
+            if ( ! is_collapsed ) {
+              xulHelper.highlightNode(d, p.highlight, false);
+            } else {
+              xulHelper.unhighlightNode(d, false);
+            }
           }
         }
       }
     }
   }
-  if ( this.nodebox )
-    this.rehighlightDesignerHits( this.nodebox.firstChild );
+  if ( this.nodebox ) {
+    if ( this.nodebox.firstChild )
+      this.rehighlightDesignerHits( this.nodebox.firstChild );
+    else logger.debug("highlightHits: nodebox is empty, can not rehighlight!");
+  }
   else
-    logger.debug("No nodebox, can not highlight");
+    logger.debug("highlightHits: No nodebox, can not highlight");
 }; // highlightHits
 
 // Rehighlight those hits on the page that correspond to the hits selected in the
@@ -1945,7 +2202,7 @@ ParseXpattern.prototype.rehighlightDesignerHits = function ( line ) {
   while (line) {
     if ( line.cf_lineType == "group" ) {
       this.rehighlightDesignerHits( line.cf_contentbox.firstChild );
-    } else  if ( line.cf_domNode ) {
+    } else  if ( line.cf_domNode && ! line.cf_collapsed ) {
       xulHelper.highlightNode( line.cf_domNode, line.cf_highlight, true );
     }
     line = line.nextSibling;
