@@ -3,8 +3,10 @@ const DEBUG = true;
 Components.utils.import("resource://indexdata/ui/app.js");
 Components.utils.import("resource://indexdata/util/logging.js");
 Components.utils.import("resource://indexdata/ui/testEditor.js");
+Components.utils.import("resource://indexdata/ui/StepTree.js");
 Components.utils.import("resource://indexdata/runtime/ConnectorTemplate.js");
 Components.utils.import("resource://indexdata/runtime/Connector.js");
+Components.utils.import("resource://indexdata/runtime/Task.js");
 Components.utils.import("resource://indexdata/runtime/Tester.js");
 Components.utils.import("resource://indexdata/util/mozHelper.js");
 Components.utils.import("resource://indexdata/util/xmlHelper.js");
@@ -15,6 +17,12 @@ Components.utils.import("resource://indexdata/util/io.js");
 const HTMLNS = "http://www.w3.org/1999/xhtml";
 const NS = "http://www.mozilla.org/keymaster/gatekeeper/there.is.only.xul";
 
+
+//save some typing
+var child = xmlHelper.appendNode;
+var empty = xmlHelper.emptyChildren;
+
+
 //redirect 'test' logging to runMeter, carefull since at times run meter
 //may not exist, also filter DEBUG messages
 logging.getLogger('test_execution').addAppender({
@@ -41,12 +49,14 @@ app.subscribe("canSave", function (canSave) {
 
 app.subscribe("lastOpened", function (lastOpened) {
     logger.debug("taskPane: lastOpened file name changed");
-    taskPane.setSidebarTitle();
+    // only applies if task pane was open, otherwise wait until onTaskPaneLoad
+    if (app.isSidebarOn) taskPane.setSidebarTitle();
 });
 
 app.subscribe("testNameChanged", function () {
     logger.debug("Test name changed notified");
-    taskPane.refreshTaskTests();
+    // only applies if task pane was open, otherwise wait until onTaskPaneLoad
+    if (app.isSidebarOn) taskPane.refreshTaskTests();
 });
 
 app.subscribe("fullqueryEdited", function (editedValue) {
@@ -82,8 +92,10 @@ app.subscribe("selectedTask", function (selectedTask) {
         }
       );
     }
-    if (app.isSidebarOn)
+    if (app.isSidebarOn) {
+      taskPane.stepTree.setTask(app.selectedTask);
       taskPane.refreshTaskPane();
+    }
 });
 
 app.subscribe("connector", function (connector) {
@@ -93,12 +105,13 @@ app.subscribe("connector", function (connector) {
     connector
       .setStepParametersUpdateCb(taskPane.onStepParametersUpdate);
     //new connector, old selected task is invalid!{
-     var tasks = app.connector.findTasks(
+    var tasks = app.connector.findTasks(
       app.connector.template.properties["defaultTaskSelection"]);
-    if (!tasks) {
+    if (!tasks && app.isSidebarOn) {
       logger.debug('Cannot find default task to select, try refresh with null');
       app.set("selectedTask", null);
     } else {
+      logger.debug('Selected task changed to ' + tasks[0].name);
       app.set("selectedTask", tasks[0]);
     }
 });
@@ -115,7 +128,7 @@ var prompts = Components.classes["@mozilla.org/embedcomp/prompt-service;1"]
 var taskPane = {
   // wait for the main window
   tpDoc: null,
-  stepList: null,
+  stepTree: null,
   stepBox: null,
   stepSplit: null,
   taskMenu: null,
@@ -126,53 +139,6 @@ var taskPane = {
   dataTab: 1,
   stepClip: [],
 
-  cutStep: function (stepList, task) {
-    taskPane.stepClip = [];
-    var items = taskPane.getItemsInOrder(stepList, false);
-    for (var i=0; i<items.length; i++) {
-      var index = items[i].value;
-      taskPane.stepClip.unshift(task.deleteItem(index));
-    }
-  },
-
-  copyStep: function (stepList, task) {
-    taskPane.stepClip = [];
-    var items = taskPane.getItemsInOrder(stepList, true);
-    for (var i=0; i<items.length; i++) {
-      var index = items[i].value;
-      taskPane.stepClip.push(task.steps[index].clone());
-    }
-  },
-
-  disableSelected: function (stepList, task) {
-    var stepList = taskPane.stepList;
-    for (var i=0; i<stepList.selectedItems.length; i++) {
-      var index = stepList.selectedItems[i].value;
-      var step = app.selectedTask.steps[index];
-      if (step.getClassName() === "Comment") break;
-      var disabled = step.isDisabled == true;
-      step.isDisabled = !disabled;
-    }
-    taskPane.refreshSteps();
-  },
-
-  pasteStep: function (stepList, task) {
-    var index = stepList.selectedIndex;
-    index = index < 0 ? 0 : index;
-    if (taskPane.stepClip.length == 0) {
-      logger.debug("Cannot paste, buffer empty");
-      return;
-    }
-    for (var i=0; i<taskPane.stepClip.length; i++) {
-      var clone = taskPane.stepClip[i].clone();
-      task.insertItem(clone, index);
-      if (clone.getClassName() !== "Comment") task.attachStep(clone);
-      index++;
-    }
-    taskPane.refreshSteps();
-    app.set("canSave", true);
-  },
-
   setDisabledTestControls: function (disable) {
     taskPane.tpDoc.getElementById('cfTaskTests').disabled = disable;
     //taskPane.tpDoc.getElementById('cfTestAdd').disabled = disable;
@@ -188,7 +154,7 @@ var taskPane = {
   onStepParametersUpdate: function(step) {
     logger.debug("step parameters updated for " + step );
     app.set("canSave", true);
-    taskPane.refreshSteps();
+    taskPane.stepTree.refresh();
   },
 
   onNewTaskLoad: function (e) {
@@ -238,13 +204,40 @@ var taskPane = {
     taskPane.tpDoc = e.target;
     app.set("mainWindow", e.currentTarget.top);
     taskPane.mainWindow = e.currentTarget.top;
-
+    
     // and some commonly referenced elements
-    taskPane.stepList = taskPane.tpDoc.getElementById("cfStepList");
     taskPane.stepBox = taskPane.mainWindow.document.getElementById("cfStepBox");
     taskPane.stepSplit = taskPane.mainWindow.document.getElementById("cfContentSplitter");
     taskPane.taskMenu = taskPane.tpDoc.getElementById("cfTaskMenu");
 
+    // set up the step tree
+    let stepTreeElement = taskPane.tpDoc.getElementById("cfStepTree")
+    taskPane.stepTree = new StepTree();
+    taskPane.stepTree.init(stepTreeElement, {
+      onChange: function () { app.set("canSave", true); }, 
+      onSelect: function (item, index) { 
+        empty(taskPane.stepBox);
+        if (item.getClassName() != "Block") {
+          item.draw(taskPane.stepBox, taskPane.mainWindow);
+          taskPane.stepSplit.setAttribute("collapsed", "false");
+          taskPane.stepBox.setAttribute("collapsed", "false");
+        } else {
+          taskPane.stepSplit.setAttribute("collapsed", "true");
+          taskPane.stepBox.setAttribute("collapsed", "true");
+        }
+        taskPane.selectedStep = item;
+        app.selectedTask.selectedStepIndex = index;
+        app.notify("selectedStep");
+      },
+      onDeselect: function () {
+        // No steps at the moment, hide step box.
+        taskPane.stepBox.setAttribute("collapsed", "true");
+        taskPane.stepSplit.setAttribute("collapsed", "true");
+        return;
+      }
+    });
+
+
     //check licence and disallow the task pane if not accepted.
     //if so disable the builder
     var isLicenceAccepted = function () {
@@ -317,7 +310,8 @@ var taskPane = {
 
   onTaskSelect: function (t) {
     app.selectedTask = app.connector.getTaskById(t);
-    if (typeof (app.selectedTask.selectedStepIndex) === 'undefined') app.selectedTask.selectedStepIndex = 0;
+    if (typeof (app.selectedTask.selectedStepIndex) === 'undefined') 
+      app.selectedTask.selectedStepIndex = 0;
     taskPane.selectedStep = null;
     app.notify("selectedTask");
   },
@@ -344,9 +338,10 @@ var taskPane = {
   onRunAll: function() {
     let task = app.selectedTask;
     taskPane.bindToRunMeter();
-    for (let i=0; i < task.steps.length; i++) {
-      if (typeof task.steps[i].processConf === "function")
-        task.steps[i].processConf();
+    //FIXME this does not handle blocks
+    for (let i=0; i < task.block.getItemsSize(); i++) {
+      if (typeof task.block.getItemAt(i).processConf === "function")
+        task.block.getItemAt(i).processConf();
     }
     task.clearData();
     taskPane.prepTask(task);
@@ -354,11 +349,7 @@ var taskPane = {
   },
 
   highlightError: function (pos) {
-    var list = taskPane.tpDoc.getElementById("cfStepList");
-    list.ensureIndexIsVisible(pos);
-    list.suppressOnSelect = true;
-    list.selectedIndex = pos;
-    list.suppressOnSelect = false;
+    //TODO not implemented
   },
 
   bindToRunMeter: function () {
@@ -366,7 +357,6 @@ var taskPane = {
     taskPane.stepBox.setAttribute("collapsed", "false");
     taskPane.runMeter = taskPane.drawRunMeter(taskPane.stepBox);
     taskPane.selectedStep = null;
-    taskPane.tpDoc.getElementById('cfStepList').selectedIndex = -1;
     app.selectedTask.addHandler("onStepFinish", taskPane.updateMeterProgress);
     app.selectedTask.addHandler("onMessageLog", taskPane.updateMeterLog);
     app.selectedTask.addHandler("onTaskFinish", taskPane.updateMeterOutcome);
@@ -390,24 +380,28 @@ var taskPane = {
   },
 
   onRunSelected: function () {
-    let task = app.selectedTask;
-    let items = taskPane.getItemsInOrder(taskPane.stepList, true);
-    let len = items.length;
-    for (let i=0; i<len; i++) {
-      let from = Number(items[i].value);
-      let to = from;
-      while (i<len-1 && Number(items[i+1].value)-to == 1) {
-        to = Number(items[i+1].value);
-        i++;
+    let items = taskPane.stepTree.getValidSelection()[1];
+    let from = items.length > 0 ? items[0].block.findItemIndex(items[0]) : 0;
+    for (let i=0; i<items.length; i++) {
+      let nextItem = i+1 < items.length ? items[i+1] : null;
+      let item = items[i];
+      let block = item.block;
+      let index = block.findItemIndex(item);
+      //what is this process conf?
+      if (typeof item.processConf === "function") item.processConf();
+      if (nextItem == null //eos
+        || nextItem.block != item.block //we got to the new block
+        || index - to > 1) { //or next selection
+        let step = block.getItemAt(from);
+        let switched = false;
+        if (step.isAlt === true) { step.isAlt = false; switched = true };
+        logger.info("About to run steps from "+from+" to "+index);
+        taskPane.prepTask(app.selectedTask);
+        block.run(null, from, index+1);
+        if (switched) step.isAlt = true;
+        //move cursor
+        from = index;
       }
-      let step = task.steps[from];
-      if (typeof step.processConf === "function") step.processConf();
-      let switched = false;
-      if (step.isAlt === true) { step.isAlt = false; switched = true };
-      logger.info("About to run steps from "+from+" to "+to);
-      taskPane.prepTask(task);
-      task.run(from, to+1);
-      if (switched) step.isAlt = true;
     }
   },
 
@@ -415,44 +409,6 @@ var taskPane = {
     taskPane.mainWindow.alert(errorMsg);
   },
 
-  /*
-   * Experiment shows that this gets called a lot with no selection
-   * (i.e. selectedIndex == -1, selectedItem === null).  For example,m
-   * this happens twice on startup, before we've done anything at all.
-   * We need to be prepared to ignore those invocations for now,
-   * though eventually it would be better to properly understand why
-   * they're happening.
-   */
-  onStepSelected: function (list) {
-    logger.debug("in onStepSelected(), selectedIndex='" + list.selectedIndex + "', selectedItem='" + list.selectedItem + "'");
-    if (list.selectedItem) {
-      /// Previously, selIndex was set to list.selectedItem.value, but
-      /// this does not work reliably, e.g. when scrolled out of the
-      /// window.  In these circumstances, list.selectedItem is
-      /// defined, but none of its properties seem to be.
-      var selIndex = list.selectedIndex;
-      app.selectedTask.selectedStepIndex = selIndex;
-      var selStep = app.selectedTask.steps[selIndex];
-      //*logger.debug("onStepSelected(), item='" + list.selectedIndex + "', step " + selIndex + " = " + selStep );
-      if (selStep === undefined) {
-        //*logger.debug("This is a clue that something has gone wrong");
-        return;
-      }
-      if (selStep === taskPane.selectedStep) return;
-      taskPane.selectedStep = selStep;
-      logger.debug("onStepSelected() calling editStep()");
-      taskPane.editStep(selStep);
-      app.notify("selectedStep");
-    }
-  },
-
-  markAsAlt: function (stepItems, bool) {
-    for (var i=0; i<stepItems.length; i++) {
-        app.selectedTask.steps[stepItems[i].value].isAlt = bool;
-    }
-    taskPane.refreshSteps();
-  },
-
   removeCurrentTask: function () {
     if (app.selectedTask) {
       if (prompts.confirm(null, "Please confirm",
@@ -545,82 +501,6 @@ var taskPane = {
     node.setAttribute("tooltiptext", string);
   },
 
-  newComment: function () { 
-    let index = taskPane.stepList.selectedIndex;
-    taskPane.refreshSteps(app.selectedTask.addComment("", index));
-    app.set("canSave", true);
-  },
-
-  newStep: function (step_name) {
-    let index = taskPane.stepList.selectedIndex;
-    taskPane.refreshSteps(app.selectedTask.addStep(step_name, index));
-    app.set("canSave", true);
-    // no need to call editStep() here: refresh assigns the
-    // selectedIndex. that causes onStepSelected() to fire,
-    // which calls editStep().
-  },
-
-  editStep: function (step) {
-    //*logger.debug("in taskPane.editStep()");
-    xmlHelper.emptyChildren(taskPane.stepBox);
-    step.draw(taskPane.stepBox, taskPane.mainWindow);
-    taskPane.stepSplit.setAttribute("collapsed", "false");
-    taskPane.stepBox.setAttribute("collapsed", "false");
-  },
-
-  deleteSteps: function (stepItems) {
-    var stepNums = []; // collect indices
-    for (var i=0; i<stepItems.length; i++) {
-        stepNums.push(stepItems[i].value);
-    }
-    app.selectedTask.deleteItems(stepNums);
-    taskPane.refreshSteps();
-    app.set("canSave", true);
-  },
-
-  getItemsInOrder: function(stepList, incr) {
-    var items = [];
-    var len = stepList.selectedItems.length;
-    //copy
-    for (var i=0; i<len; i++) {
-      items.push(stepList.selectedItems[i]);
-    }
-    //sort
-    items.sort(function(a,b) {
-        return incr
-          ? stepList.getIndexOfItem(a) - stepList.getIndexOfItem(b)
-          : stepList.getIndexOfItem(b) - stepList.getIndexOfItem(a);
-    });
-    return items;
-  },
-
-  stepMove: function(up /*up true*/) {
-    var stepList = taskPane.stepList;
-    var sel = [];
-    var items = taskPane.getItemsInOrder(stepList, up);
-    var len = items.length;
-    for (var i=0; i<len; i++) {
-      var index = stepList.getIndexOfItem(items[i]);
-      if (up && index > i) {
-        app.selectedTask.itemUp(index);
-        sel.push(index-1);
-      } else if (!up && (index+1+i
-          < app.selectedTask.steps.length)) {
-        app.selectedTask.itemDown(index);
-        sel.push(index+1);
-      } else {
-        sel.push(index);
-      }
-    }
-    taskPane.refreshSteps();
-    stepList.clearSelection();
-    for (var i=0; i<sel.length; i++) {
-      var item = stepList.getItemAtIndex(sel[i]);
-      stepList.addItemToSelection(item);
-    }
-    app.set("canSave", true);
-  },
-
   runCurrentTest: function () {
     if (testEditor.getActiveTest() == null)
       logger.debug("Current test is null");
@@ -703,7 +583,7 @@ var taskPane = {
     taskPane.refreshTaskMenu();
     taskPane.refreshArguments();
     taskPane.refreshData();
-    taskPane.refreshSteps();
+    taskPane.stepTree.refresh();
   },
 
   /*
@@ -801,7 +681,7 @@ var taskPane = {
 
   refreshData: function () {
     let task = app.selectedTask;
-    let keys = Object.keys(task.data);
+    let keys = Task.dataKeys.concat(Task.connectorDataKeys);
     let tabBox = taskPane.tpDoc.getElementById("cfDataBrowser");
     xmlHelper.emptyChildren(tabBox);
 
@@ -810,11 +690,16 @@ var taskPane = {
       let tab = xmlHelper.appendNode(tabs, "tab", null, {label: keys[i]});
     }
 
-    let panels = xmlHelper.appendNode(tabBox, "tabpanels", null, { flex: 1 });
+    let panels = xmlHelper.appendNode(tabBox, "tabpanels", null, { flex: 1, style: 'padding: 0; margin: 0;'  });
     for (let i = 0; i < keys.length; i++) {
-      let panel = xmlHelper.appendNode(panels, "tabpanel", null, { flex: 1 });
+      let panel = xmlHelper.appendNode(panels, "tabpanel", null, { flex: 1, style: 'padding: 0; margin: 0;' });
 
-      let tree = xmlHelper.appendNode(panel, "tree", null, {flex: 1, value: keys[i]});
+      let tree = xmlHelper.appendNode(panel, "tree", null, {
+        flex: 1,
+        value: keys[i],
+        hidecolumnpicker: 'true',
+        style: 'padding: 0; margin: 0;'
+      });
       // couldn't get the context attribute to work, seems like all this is still necessary
       tree.addEventListener("contextmenu", taskPane.onDataTreeClick, false);
 
@@ -829,15 +714,18 @@ var taskPane = {
     tabBox.selectedIndex = taskPane.dataTab;
   },
 
-  addDataNodes: function (root, obj, parentPath) {
+  addDataNodes: function (root, obj, parentPath, parentName) {
     for (key in obj) {
       // TODO: generate JSONpath here
       var jsPath = parentPath + "/" + key;
       var item = xmlHelper.appendNode(root, "treeitem", null, {"assertpath": jsPath});
       var row = xmlHelper.appendNode(item, "treerow");
       var dispkey = key;
-      if ( Array.isArray(obj) && obj.length )
-        dispkey = key + " of " + obj.length ;
+      if ( parentName && ! parentName.match( /^[0-9]+/ ) )
+        dispkey = parentName + " [" + key + "]";
+      if ( typeof(obj[key]) == "object" && obj[key].length > 1 ) {
+        dispkey = key + " (" + obj[key].length + ")";
+      }
       //xmlHelper.appendNode(row, "treecell", null, {"label": key });
       xmlHelper.appendNode(row, "treecell", null, {"label": dispkey });
       if (typeof obj[key] != "object") {
@@ -851,9 +739,9 @@ var taskPane = {
         item.setAttribute("open", "true");
         var crib = xmlHelper.appendNode(item, "treechildren");
         if (obj[key].length == 1) {
-          taskPane.addDataNodes(crib, obj[key][0], jsPath);
+          taskPane.addDataNodes(crib, obj[key][0], jsPath, "");
         } else {
-          taskPane.addDataNodes(crib, obj[key], jsPath);
+          taskPane.addDataNodes(crib, obj[key], jsPath, key);
         }
       }
     }
@@ -889,141 +777,6 @@ var taskPane = {
     }
   },
 
-  /*
-   * Experiment shows that the line
-   *    xmlHelper.emptyChildren(list, "listitem");
-   * below sometimes (but now always) causes the onStepSelected()
-   * handler to be invoked twice -- never more, never less.  In these
-   * invocations, selectedIndex is -1, so they are easily discarded;
-   * but we need to know these things to properly understand the flow
-   * of control through all the handlers.
-   */
-  refreshSteps: function (indexToSelect) {
-    var list = taskPane.tpDoc.getElementById("cfStepList");
-    if (indexToSelect === undefined) {
-      if (typeof(app.selectedTask.selectedStepIndex) !== 'undefined') indexToSelect = app.selectedTask.selectedStepIndex;
-      else if (list.selectedIndex !== -1) indexToSelect = list.selectedIndex;
-      else indexToSelect = 0;
-    }
-    xmlHelper.emptyChildren(list, "listitem");
-    xmlHelper.emptyChildren(list, "separator");
-    var task = app.selectedTask;
-    if (task == null) {
-      logger.debug("selectedTask is null, ignore refreshing steps");
-      return;
-    }
-    if (task.steps.length < 1) {
-      // No steps at the moment, hide step box.
-      taskPane.stepBox.setAttribute("collapsed", "true");
-      taskPane.stepSplit.setAttribute("collapsed", "true");
-      return;
-    }
-    //initialize indent for each step
-    var indent = [];
-    for (var i=0; i<task.steps.length; i++) {
-      indent[i] = 0;
-    }
-    for (var i=0; i<task.steps.length; i++) {
-      var step = task.steps[i];
-      // listitem has to be appended AFTER its children!
-      // the value is st to the index of the step, DO NOT CHANGE THIS
-      if (step.getIndentRange) {
-        var ir = step.getIndentRange();
-        if (ir[1] < 0) ir[1] = task.steps.length;
-        var level = indent[i]+1;
-        if (step.isAlt) {
-          level++;
-          indent[i]++;
-        }
-        for (var j = ir[0]; j<ir[1]; j++) {
-          indent[i+j] += level;
-        }
-      }
-      var item = xmlHelper.appendNode(list, "listitem", null,
-          { value: i, allowevents: true,
-            context: "stepListItemContextMenu",
-            indentLevel: indent[i]}, null, false);
-      if (step.getClassName() === "Comment") {
-        taskPane.drawCommentIntoNode(step, i, item);
-      } else {
-        taskPane.drawStepIntoNode(step, i, item, task, indent[i], taskPane.refreshSteps);
-      }
-      list.appendChild(item);
-    }
-
-    var n = list.itemCount;
-    if (indexToSelect >= n) {
-      logger.debug("Reducing indexToSelect from " + indexToSelect +
-          " to " + (n-1) );
-      indexToSelect = n-1;
-    }
-    // Avoid doing this if unnecessary, to avert an event cascade
-    if (list.selectedIndex != indexToSelect) {
-      // it defaults to just ensuring a few pixels are visible;
-      // let's put it in the middle
-      let indexToExpose = indexToSelect + Math.round(list.getNumberOfVisibleRows()/2);
-      if (indexToExpose >= n) indexToExpose = n - 1;
-      list.ensureIndexIsVisible(indexToExpose);
-      list.selectedIndex = indexToSelect;
-    }
-  },
-
-  drawCommentIntoNode: function (step, index, item) {
-    let commentText = " // " + step.body;
-    let style = "cursor:default; overflow:hidden;";
-    xmlHelper.appendNode(item, "listcell", null, { "label": index});
-    xmlHelper.appendNode(item, "listcell", null,
-      {"style": style, "tooltiptext": step.body, "label": commentText});
-  },
-
-  drawStepIntoNode: function (step, index, item, task, indent, refreshCb) {
-    xmlHelper.appendNode(item, "listcell", null, { "label": index,
-        disabled: disabled});
-    var disabled = step.isDisabled == true;
-    var tooltip = step.getComments().join("; ");
-    var style =  "cursor:default; overflow:hidden;";
-    if (step.isDeprecated()) {
-      style += " color:red;";
-      tooltip = "DEPRECATED - " + step.getDeprecationInfo();
-    }
-    if (disabled) {
-      style += " color:grey; font-style:italic;";
-    }
-    var labelcell = xmlHelper.appendNode(item, "listcell", null,
-      {"style": style, 'tooltiptext': tooltip, disabled: disabled},
-      null, false);
-
-    xmlHelper.appendNode(labelcell, "span", "\u00A0",
-        null, HTMLNS);
-    for (var i=0; i<indent; i++) {
-       xmlHelper.appendNode(labelcell, "span", "\u00B7 ",
-        null, HTMLNS);
-    }
-
-    xmlHelper.appendNode(labelcell, "b", step.getDisplayName(), null, HTMLNS);
-
-    // ### This causes warnings in modern xulrunners:
-    // XUL box for listcell element contained an inline b child,
-    // forcing all its children to be wrapped in a block
-    // The same happens to the span below, if this is commented out.
-    if (step.renderArgs)
-      xmlHelper.appendNode(labelcell, "span",
-            " " + step.renderArgs(), null, HTMLNS);
-    item.appendChild(labelcell);
-    var cb = xmlHelper.appendNode(item, "listcell", null,
-        {"type": "checkbox", "checked": step.isAlt, "value": index,
-        disabled: disabled});
-    if (!disabled)
-      cb.addEventListener("click", function(e) {
-        var i = this.getAttribute("value");
-        task.steps[i].isAlt = !task.steps[i].isAlt;
-        app.set("canSave", true);
-        refreshCb();
-        //e.stopPropagation(); // doesn't prevent step being selected.
-      }, false);
-    if (disabled) item.disabled = true;
-  },
-
   refreshTaskTests: function () {
     var menuList = taskPane.tpDoc.getElementById("cfTaskTests");
     var popup = menuList.getElementsByTagName("menupopup")[0];
@@ -1049,9 +802,3 @@ var taskPane = {
     taskPane.tpDoc.getElementById("cfSaveButton").disabled = true;
   }
 };
-
-// Local variables:
-// c-basic-offset: 2
-// indent-tabs-mode: nil
-// End:
-// vim: shiftwidth=2 tabstop=8 expandtab
