@@ -956,17 +956,54 @@ xulHelper = {
     if (!attrs )
         attrs = {};
     if (label )
-        attrs.label = label;    
+        attrs.label = label;
     var menu = xmlHelper.appendNode(surface, "menu", null, attrs );
-                                        //{ label:label } );
     var pop = xmlHelper.appendNode(menu, "menupopup", null, null );
+    pop.cf_menu = menu; // keep a reference, so we can change the label
     return pop;
   },
 
+  // Builds a label and a popup menu for it
+  // Popup menus need a unique id, so we keep a global counter here
+  nextpopupmenuid: 1,
+  
+  popupmenu: function (surface, label, attrs) {
+    var menuid = "popupmenu" + xulHelper.nextpopupmenuid++ ;
+    logger.debug("Popup menu id=" + menuid);
+    var popupset = xmlHelper.appendNode(surface, "popupset" );
+    var pop = xmlHelper.appendNode(popupset, "menupopup", null,
+                                      { id:menuid } );
+    if (!attrs )
+        attrs = {};
+    attrs.popup = menuid;
+    var disp = xmlHelper.appendNode(surface, "label", label, attrs);
+    pop.cf_label = disp;
+    return pop;
+  },
+  
+  // Change the label of a menu. Works for things created with menu or
+  // popupmenu above.
+  changemenulabel: function( menupop, label) {
+    if ( !menupop ) {
+      logger.debug("xulHelper: changemenulabel called with no menu " +
+        "(and label '" + label + "'). Can not change");
+    }
+    if ( menupop.cf_menu )
+      menupop.cf_menu.label = label;
+    else if ( menupop.cf_label )
+      menupop.cf_label.value = label;
+    else      
+      logger.debug("xulHelper: changemenulabel: no cf_menu in menupopup, " +
+        "can not change label to '" + label + "'");
+  },
+  
   // Adds a menu item into the menu
-  menuitem: function( menu, label) {
-    var item = xmlHelper.appendNode(menu, "menuitem", null,
-                                           { label: label } );
+  menuitem: function( menu, label, attrs) {
+    if ( !attrs )
+      attrs = {};
+    if ( label && ! attrs.label )
+      attrs.label = label;
+    var item = xmlHelper.appendNode(menu, "menuitem", null, attrs);
     return item;
   },
 
