@@ -4,7 +4,7 @@
 DIR=`dirname $0`            # ../../connectors/cf-zserver-regression
 PLAINDIR=`basename $DIR`    # cf-zserver-regression
 
-echo "running mpr-egression in $DIR"
+echo "running $0"
 GRET=0
 : ${diff_opt="-u"}
 : ${REGRESSION_FATAL=false}
